class MinHeap:
    def __init__(self):
        self.heap_list = []
        self.current_size = 0

    def insert(self, k):
        self.heap_list.append(k)
        self.current_size += 1
        self.sift_up(self.current_size - 1)

    def top(self):
        if self.current_size == 0:
            return None
        return self.heap_list[0]

    def delete_min(self):
        assert len(self.heap_list) > 0

        root = self.heap_list[0]
        self.heap_list[0] = self.heap_list[-1]
        del self.heap_list[-1]
        self.current_size -= 1
        self.sift_down(0)

        return root

    def sift_down(self, i):
        while i * 2 + 1 < self.current_size:
            mc = self.min_child(i)
            if self.heap_list[i] > self.heap_list[mc]:
                self.heap_list[i], self.heap_list[mc] = self.heap_list[mc], self.heap_list[i]
            i = mc

    def sift_up(self, i):
        while i > 0:
            p = (i - 1) // 2
            if self.heap_list[i] >= self.heap_list[p]:
                break
            self.heap_list[i], self.heap_list[p] = self.heap_list[p], self.heap_list[i]
            i = p

    def min_child(self, i):
        if (i * 2) + 2 >= self.current_size:
            return i * 2 + 1
        if self.heap_list[i * 2 + 1] < self.heap_list[i * 2 + 2]:
            return i * 2 + 1
        else:
            return i * 2 + 2


def main():
    n = int(input())

    a = [(0, 0, 0)] * n
    for i in range(n):
        l, r = [int(v) for v in input().split()]
        a[i] = (l, r, i)

    a.sort(reverse=True)
    h = MinHeap()
    placements = [0] * n

    for i in range(1, n + 1):
        while len(a) > 0 and a[-1][0] <= i:
            q = a[-1]
            del a[-1]
            h.insert((q[1], q[0], q[2]))

        if h.top() is None:
            print("NO")
            return
        t = h.delete_min()
        if t[1] > i or t[0] < i:
            print("NO")
            return
        placements[i - 1] = t[2] + 1

    print("YES")
    print(*placements)


if __name__ == "__main__":
    main()
