#include <functional>
#include <algorithm>
#include "testlib.h"

using namespace std;

const int max_n = 100'000;
const int max_ab = 1'000'000'000;

class DSU {
    vector<int> p, w;
public:
    DSU(int n) : p(n + 1), w(n + 1, 0) {
        for (int i = 1; i <= n; ++i) {
            p[i] = i;
        }
    }

    int find(int v) {
        while (p[v] != v) {
            v = p[v];
        }
        return v;
    }

    void join(int a, int b) {
        a = find(a);
        b = find(b);
        if (a == b) return;
        if (w[a] == w[b]) w[a]++;
        if (w[a] < w[b]) p[a] = b;
        else p[b] = a;
    }
};

int main(int argc, char *argv[]) {
    registerValidation(argc, argv);

    int n = inf.readInt(1, max_n, "n");
    inf.readSpace();
    inf.readInt(1, n, "s");
    inf.readEoln();

    inf.readInts(n, 1, max_ab, "a_i");
    inf.readEoln();
    inf.readInts(n, 1, max_ab, "b_i");
    inf.readEoln();

    DSU dsu(n + 1);

    for (int i = 0; i < n - 1; ++i) {
        int u = inf.readInt(1, n, "u_i");
        inf.readSpace();
        int v = inf.readInt(1, n, "v_i");
        inf.readEoln();

        ensuref(u != v, "should not be loop in tree (node %d)", u);
        ensuref(dsu.find(u) != dsu.find(v), "should not be even or cycle making edges (node %d)", u);
        dsu.join(u, v);
    }

    inf.readEof();
}