#include <bits/stdc++.h>
#include <variant>
#include "testlib.h"

using namespace std;

using integer = unsigned long long;
using var = string;
using value = variant<var, integer>;

#ifndef LOCAL
bool starts_with(string const &s, string const &prefix) {
    if (prefix.size() > s.size()) return false;
    for (int i = 0; i < prefix.size(); i++) {
        if (s[i] != prefix[i]) return false;
    }
    return true;
}

bool ends_with(string const &s, string const &suffix) {
    if (suffix.size() > s.size()) return false;
    for (int i = 0; i < suffix.size(); i++) {
        if (s[s.size() - i - 1] != suffix[suffix.size() - i - 1]) return false;
    }
    return true;
}
#else
bool starts_with(string const &s, string const &prefix) {
    return s.starts_with(prefix);
}

bool ends_with(string const &s, string const &suffix) {
    return s.ends_with(suffix);
}
#endif

vector<string> split_normal(string s, string const &del) {
    vector<string> result;
    while (true) {
        int pos = s.find(del);
        result.push_back(s.substr(0, pos));
        if (pos == s.npos) break;
        s.erase(0, pos + del.size());
    }
    return result;
}

int n;
vector<string> code;
vector<integer> input;

namespace origin {

    value parse_value(string const &s) {
        if ('0' <= s[0] && s[0] <= '9') {
            integer v = atoll(s.c_str());
            return value(v);
        }
        return value(s);
    }

    struct context {
        unordered_map<var, integer> var_values;
        vector<integer> output;
    };

    integer get_value(value v, context const &cxt) {
        if (holds_alternative<integer>(v)) {
            return get<integer>(v);
        }
        return cxt.var_values.at(get<var>(v));
    }

    enum struct expr_op_t {
        add = 1,
        sub = 2,
        none = 0
    };

    struct expression {
        value left, right;
        expr_op_t op;

        integer evaluate(context const &cxt) const {
            integer v1 = get_value(left, cxt);
            integer v2 = get_value(right, cxt);
            if (op == expr_op_t::add) {
                return v1 + v2;
            }
            if (op == expr_op_t::sub) {
                return v1 - v2;
            }
            return v1;
        }
    };

    struct executable {
        virtual void execute(context &cxt) const = 0;
    };

    using exec_t = shared_ptr<executable>;

    struct assignment : public executable {
        var left;
        expression right;

        assignment(var &&left, expression &&right) : left(left), right(right) {}

        void execute(context &cxt) const override {
            cxt.var_values[left] = right.evaluate(cxt);
        }
    };

    struct loop : public executable {
        var iter;
        value low, high;
        vector<exec_t> body;

        loop(var &&iter, value &&low, value &&high, vector<exec_t> &&body) :
                iter(iter), low(low), high(high), body(body) {}

        void execute(context &cxt) const override {
            integer from = get_value(low, cxt);
            integer to = get_value(high, cxt);
            for (int i = from; i <= to; i++) {
                cxt.var_values[iter] = i;
                for (auto &statement : body) {
                    statement->execute(cxt);
                }
            }
        }
    };

    struct print : public executable {
        var arg;

        explicit print(var &&arg) : arg(arg) {}

        void execute(context &cxt) const override {
            cxt.output.push_back(get_value(arg, cxt));
        }
    };

    struct read : public executable {
        var arg;

        explicit read(var &&arg) : arg(arg) {}

        void execute(context &cxt) const override {
            cxt.var_values[arg] = rnd.next(0, 2);
            input.push_back(cxt.var_values[arg]);
        }
    };

    vector<exec_t> parse(int &it) {
        vector<exec_t> program;
        while (it <= n) {
            if (it == n || ends_with(code[it], "}")) {
                it++;
                return program;
            }
            string line = code[it++];
            line.erase(0, line.find_first_not_of(' '));
            if (starts_with(line, "for ")) {
                line.erase(0, 4);
                auto items = split_normal(line, " = ");
                items[1].erase(items[1].length() - 2);
                auto values = split_normal(items[1], "...");
                auto body = parse(it);
                program.push_back(make_shared<loop>(
                        std::move(items[0]),
                        std::move(parse_value(values[0])),
                        std::move(parse_value(values[1])),
                        std::move(body)
                ));
            } else if (line.find('=') != line.npos) {
                auto items = split_normal(line, " = ");
                expression expr;
                if (items[1].find('+') != line.npos) {
                    auto values = split_normal(items[1], " + ");
                    expr = expression{parse_value(values[0]), parse_value(values[1]), expr_op_t::add};
                } else if (items[1].find('-') != line.npos) {
                    auto values = split_normal(items[1], " - ");
                    expr = expression{parse_value(values[0]), parse_value(values[1]), expr_op_t::sub};
                } else {
                    value v = parse_value(items[1]);
                    expr = expression{v, value(0ull), expr_op_t::none};
                }
                program.push_back(make_shared<assignment>(
                        std::move(items[0]),
                        std::move(expr)
                ));
            } else {
                auto items = split_normal(line, "(");
                items[1].erase(items[1].size() - 1);
                if (items[0] == "print") {
                    program.push_back(make_shared<print>(std::move(items[1])));
                } else {
                    program.push_back(make_shared<read>(std::move(items[1])));
                }
            }
        }
        return program;
    }

}

namespace result {

    struct context;

    struct executable {
        virtual void execute(context &cxt) const = 0;
    };

    using exec_t = shared_ptr<executable>;

    struct context {
        unordered_map<var, integer> var_values;
        unordered_map<var, vector<exec_t>> macros;
        unsigned int output_id = 0, input_id = 0;

        InStream &in_stream;
        vector<integer> const &expected_output;

        context(InStream &in_stream, vector<integer> const &expected_output) :
                in_stream(in_stream), expected_output(expected_output) {}
    };

    value parse_value(string const &s, context &cxt) {
        if ('0' <= s[0] && s[0] <= '9') {
            long long v = atoll(s.c_str());
            cxt.in_stream.ensuref(
                    v <= numeric_limits<int>::max(),
                    "Constant %llu is not in allowed range", v
            );
            return value(static_cast<integer>(v));
        }
        return value(s);
    }

    integer get_value(value v, context const &cxt) {
        if (holds_alternative<integer>(v)) {
            // Testlib Regex are shite
            return get<integer>(v);
        }
        string var_name = get<var>(v);
        cxt.in_stream.ensuref(
                cxt.var_values.count(var_name),
                "Variable `%s` is referenced before assignment", var_name.c_str()
        );
        return cxt.var_values.at(var_name);
    }

    enum struct expr_op_t {
        add = 1,
        sub = 2,
        none = 0
    };

    struct expression {
        value left, right;
        expr_op_t op;

        integer evaluate(context const &cxt) const {
            integer v1 = get_value(left, cxt);
            integer v2 = get_value(right, cxt);
            if (op == expr_op_t::add) {
                return v1 + v2;
            }
            if (op == expr_op_t::sub) {
                return v1 - v2;
            }
            return v1;
        }
    };

    struct assignment : public executable {
        var left;
        expression right;

        assignment(var &&left, expression &&right) : left(left), right(right) {}

        void execute(context &cxt) const override {
            cxt.var_values[left] = right.evaluate(cxt);
        }
    };

    struct repeat : public executable {
        string macro_name;
        value times;

        repeat(string &&macro_name, value &&times) : macro_name(macro_name), times(times) {}

        void execute(context &cxt) const override {
            cxt.in_stream.ensuref(
                    cxt.macros.count(macro_name),
                    "Macro `%s` is reference but does not exist", macro_name.c_str()
            );
            int result = get_value(times, cxt);
            for (int i = 0; i < result; i++) {
                for (auto const &statement : cxt.macros[macro_name]) {
                    statement->execute(cxt);
                }
            }
        }
    };

    struct print : public executable {
        var arg;

        explicit print(var &&arg) : arg(arg) {}

        void execute(context &cxt) const override {
            int v = get_value(arg, cxt);
            cxt.in_stream.ensuref(
                    cxt.output_id < cxt.expected_output.size(),
                    "Printed too much values"
            );
            cxt.in_stream.ensuref(
                    v == cxt.expected_output[cxt.output_id],
                    "Invalid number printed on position %d", cxt.output_id + 1
            );
            cxt.output_id++;
        }
    };

    struct read : public executable {
        var arg;

        explicit read(var &&arg) : arg(arg) {}

        void execute(context &cxt) const override {
            cxt.in_stream.ensuref(
                    cxt.input_id < input.size(),
                    "Tried to read too much values"
            );
            cxt.var_values[arg] = input[cxt.input_id++];
        }
    };

    string const INDENT = "    ";

    vector<pattern> const PATTERNS{
            pattern("MACRO\\ [a-z]{1,10}:"),
            pattern("REPEAT\\ [a-z]{1,10}\\ [a-z0-9]{1,10}"),
            pattern("PRINT\\ [a-z][a-z0-9]{0,9}"),
            pattern("READ\\ [a-z][a-z0-9]{0,9}"),
            pattern("[a-z][a-z0-9]{0,9}\\ GETS\\ [a-z0-9]{1,10}"),
            pattern("[a-z][a-z0-9]{0,9}\\ GETS\\ [a-z0-9]{1,10}\\ [-+]\\ [a-z0-9]{1,10}"),
    };

    void check(InStream &in, vector<integer> const &expected_output) {
        int m = in.readInt(1, 5 * n, "m");
        in.readLine();
        context cxt(in, expected_output);
        string current_macro = "";

        for (int i = 0; i < m; i++) {
            string line = in.readLine();
            if (line == "") continue;

            if (!starts_with(line, INDENT)) {
                current_macro = "";
            }
            if (PATTERNS[0].matches(line)) {
                line.erase(0, 6);
                current_macro = line.substr(0, line.size() - 1);
                in.ensuref(
                        !cxt.macros.count(current_macro),
                        "Macro with name `%s` is redefined", current_macro.c_str()
                );
            } else {
                exec_t exec;
                if (starts_with(line, INDENT)) line.erase(0, 4);
                auto items = split(line, ' ');
                if (PATTERNS[1].matches(line)) {
                    exec = make_shared<repeat>(
                            std::move(items[1]),
                            std::move(parse_value(items[2], cxt))
                    );
                } else if (PATTERNS[2].matches(line)) {
                    exec = make_shared<print>(std::move(items[1]));
                } else if (PATTERNS[3].matches(line)) {
                    exec = make_shared<read>(std::move(items[1]));
                } else if (PATTERNS[4].matches(line)) {
                    expression expr{parse_value(items[2], cxt), value(0ull), expr_op_t::none};
                    exec = make_shared<assignment>(
                            std::move(items[0]),
                            std::move(expr)
                    );
                } else if (PATTERNS[5].matches(line)) {
                    expression expr{
                            parse_value(items[2], cxt),
                            parse_value(items[4], cxt),
                            items[3] == "+" ? expr_op_t::add : expr_op_t::sub
                    };
                    exec = make_shared<assignment>(
                            std::move(items[0]),
                            std::move(expr)
                    );
                } else {
                    in.quitf(_pe, "Invalid command on line %d", i + 1);
                }
                if (current_macro != "") {
                    cxt.macros[current_macro].push_back(exec);
                } else {
                    exec->execute(cxt);
                }
            }
        }

        in.ensuref(cxt.output_id == expected_output.size(), "Printed too few values");
    }

}

int main(int argc, char *argv[]) {

    registerTestlibCmd(argc, argv);

    n = inf.readInt();
    inf.readLine();
    for (int i = 0; i < n; i++) {
        code.push_back(inf.readLine());
    }
    int it = 0;
    auto program = origin::parse(it);

    for (int test = 0; test < 5; test++) {
        input.clear();
        origin::context cxt;
        for (auto const &statement : program) {
            statement->execute(cxt);
        }

        ans.init(argv[3], TMode::_answer);
        result::check(ans, cxt.output);
        ans.close();

        ouf.init(argv[2], TMode::_output);
        result::check(ouf, cxt.output);
        ouf.close();
    }

    quitf(_ok, "all checks passed");

}